package fr.hugol;

import org.graphstream.algorithm.Toolkit;
import org.graphstream.graph.BreadthFirstIterator;
import org.graphstream.graph.Graph;
import org.graphstream.graph.Node;
import org.graphstream.graph.implementations.SingleGraph;
import org.graphstream.stream.file.FileSource;
import org.graphstream.stream.file.FileSourceEdge;

import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Locale;

public class Main {
    public static void main(String[] args) {
        System.setProperty("org.graphstream.ui", "swing");
        var g = create();

        var nodeCount = g.getNodeCount();
        var edgeCount = g.getEdgeCount();
        var avgDegree = Toolkit.averageDegree(g);
        var avgClusteringCoeff = Toolkit.averageClusteringCoefficient(g);

        // Q1
        System.out.println("\nQ1");
        System.out.println("- Nodes: " + nodeCount);
        System.out.println("- Edges: " + edgeCount);
        System.out.println("- Avg. degree: " + avgDegree);
        System.out.println("- Avg. clustering coeff.: " + avgClusteringCoeff);

        // Q2
        System.out.println("\nQ2");
        System.out.println("- Avg. clustering coeff. for random graph: " + (avgDegree / nodeCount));

        // Q3
        System.out.println("\nQ3");
        var avgDegreeForConnectedRegime = Math.log(nodeCount);
        System.out.println("- Is connected: " + Toolkit.isConnected(g));
        System.out.println("- All random graphs are connected: " + (avgDegree > avgDegreeForConnectedRegime));
        System.out.println("- Avg. degree for connected regime: " + avgDegreeForConnectedRegime);

        // Q4
        System.out.println("\nQ4");
        try {
            generateDegreeDistributionFile(g);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

        // Q5
        System.out.println("\nQ5");
        System.out.println("- Avg. distance: " + getAverageDistance(g, 1000));
    }

    public static Graph create() {
        var g = new SingleGraph("TP");

        FileSource fs = new FileSourceEdge();
        fs.addSink(g);
        try {
            fs.readAll("src/main/resources/data/com-dblp.ungraph.txt");
        } catch( IOException ignored) {
        } finally {
            fs.removeSink(g);
        }

        return g;
    }

    public static void generateDegreeDistributionFile(Graph graph) throws IOException {
        FileWriter fileWriter = new FileWriter("gnuplot/dd_dblp.dat");
        PrintWriter printWriter = new PrintWriter(fileWriter);

        int[] dd = Toolkit.degreeDistribution(graph);
        for (int k = 0; k < dd.length; k++) {
            if (dd[k] != 0) {
                printWriter.printf(Locale.US, "%6d%20.8f%n", k, (double)dd[k] / graph.getNodeCount());
            }
        }
        printWriter.close();
    }

    /**
     * Calcule la distance moyenne d'un Graphe à l'aide du parcours en largeur
     */
    public static double getAverageDistance(Graph g, int nodeCount)
    {
        long sum = 0;
        for (var node : Toolkit.randomNodeSet(g, nodeCount)) {
            var iterator = new BreadthFirstIterator(node);
            while (iterator.hasNext()) {
                sum += iterator.getDepthOf(iterator.next());
            }
        }
        return ( 1.0D / ( 1000.0D * g.getNodeCount() ) ) * (double) sum;
    }
}
